@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2016 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -20,12 +20,23 @@
 */
 #include "../../SDL_internal.h"
 
-#ifndef _SDL_x11mouse_h
-#define _SDL_x11mouse_h
+#ifndef SDL_x11mouse_h_
+#define SDL_x11mouse_h_
+
+typedef struct SDL_XInput2DeviceInfo
+{
+    int device_id;
+    SDL_bool relative[2];
+    double minval[2];
+    double maxval[2];
+    double prev_coords[2];
+    Time prev_time;
+    struct SDL_XInput2DeviceInfo *next;
+} SDL_XInput2DeviceInfo;
 
 extern void X11_InitMouse(_THIS);
 extern void X11_QuitMouse(_THIS);
 
-#endif /* _SDL_x11mouse_h */
+#endif /* SDL_x11mouse_h_ */
 
 /* vi: set ts=4 sw=4 expandtab: */
