@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 1997-2011 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -26,19 +26,22 @@ main(int argc, char *argv[])
     SDL_version compiled;
     SDL_version linked;
 
-#if SDL_VERSION_ATLEAST(1, 3, 0)
-    printf("Compiled with SDL 1.3 or newer\n");
+    /* Enable standard application logging */
+    SDL_LogSetPriority(SDL_LOG_CATEGORY_APPLICATION, SDL_LOG_PRIORITY_INFO);
+
+#if SDL_VERSION_ATLEAST(2, 0, 0)
+    SDL_Log("Compiled with SDL 2.0 or newer\n");
 #else
-    printf("Compiled with SDL older than 1.3\n");
+    SDL_Log("Compiled with SDL older than 2.0\n");
 #endif
     SDL_VERSION(&compiled);
-    printf("Compiled version: %d.%d.%d.%d (%s)\n",
+    SDL_Log("Compiled version: %d.%d.%d (%s)\n",
            compiled.major, compiled.minor, compiled.patch,
-           SDL_REVISION_NUMBER, SDL_REVISION);
+           SDL_REVISION);
     SDL_GetVersion(&linked);
-    printf("Linked version: %d.%d.%d.%d (%s)\n",
+    SDL_Log("Linked version: %d.%d.%d (%s)\n",
            linked.major, linked.minor, linked.patch,
-           SDL_GetRevisionNumber(), SDL_GetRevision());
+           SDL_GetRevision());
     SDL_Quit();
     return (0);
 }
